@@ -16,8 +16,8 @@ from DataRecorder import Recorder
 from requests import Session, Response
 from requests.structures import CaseInsensitiveDict
 
-from ._funcs import FileExistsSetter, PathSetter, BlockSizeSetter, copy_session, _set_charset, _get_file_info
-from .mission import Task, Mission, MissionData
+from ._funcs import FileExistsSetter, PathSetter, BlockSizeSetter, copy_session, _set_charset, _get_file_info, LogMode
+from .mission import Task, Mission, MissionData, BaseTask
 
 
 class DownloadKit(object):
@@ -48,6 +48,10 @@ class DownloadKit(object):
         self._interval = None
         self._timeout = None
 
+        self._print_mode = None
+        self._log_mode = None
+        self._logger = None
+
         self.goal_path: str = goal_path or '.'
         self.file_exists: str = file_exists
         self.show_errmsg: bool = False
@@ -80,6 +84,26 @@ class DownloadKit(object):
                         split=False,
                         **kwargs).wait(show=show_msg)
 
+    def set_print(self) -> LogMode:
+        """设置打印到控制台的信息，可选全部、错误任务、不打印"""
+        if self._print_mode is None:
+            self._print_mode = LogMode()
+        return self._print_mode
+
+    def set_log(self, log_path: [str, Path] = None) -> LogMode:
+        """设置记录到文件的信息，可选全部、错误任务、不记录   \n
+        :param log_path: 记录文件路径
+        :return: LogMode对象
+        """
+        if self._log_mode is None:
+            self._log_mode = LogMode()
+        if log_path is not None:
+            if self._logger is not None:
+                self._logger.set_path(log_path)
+            else:
+                self._logger = Recorder(log_path, 1)
+        return self._log_mode
+
     @property
     def roads(self) -> int:
         """可同时运行的线程数"""
@@ -88,7 +112,7 @@ class DownloadKit(object):
     @roads.setter
     def roads(self, val: int) -> None:
         """设置可同时运行的线程数"""
-        if self.is_running():
+        if self.is_running:
             print('有任务未完成时不能改变roads。')
             return
         if val != self._roads:
@@ -175,10 +199,23 @@ class DownloadKit(object):
         except ImportError:
             self._session = Session()
 
+    @property
     def is_running(self) -> bool:
-        """检查是否有线程还在运行中"""
+        """返回是否有线程还在运行"""
         return any(self._threads.values()) or not self.waiting_list.empty()
 
+    def set_proxies(self, http: str = None, https: str = None) -> None:
+        """设置代理地址及端口，例：'http://127.0.0.1:1080'         \n
+        :param http: http代理地址及端口
+        :param https: https代理地址及端口
+        :return: None
+        """
+        if not http.startswith('http://'):
+            http = f'http://{http}'
+        if not https.startswith('http'):
+            https = f'http://{https}'
+        self._session.proxies = {'http': http, 'https': https}
+
     def add(self,
             file_url: str,
             goal_path: Union[str, Path] = None,
@@ -205,12 +242,12 @@ class DownloadKit(object):
                            split=split,
                            kwargs=kwargs)
         self._missions_num += 1
-        mission = Mission(self._missions_num, data)
+        mission = Mission(self._missions_num, data, self)
         self._missions[self._missions_num] = mission
         self._run_or_wait(mission)
         return mission
 
-    def _run_or_wait(self, mission: Mission):
+    def _run_or_wait(self, mission: BaseTask):
         """接收任务，有空线程则运行，没有则进入等待队列"""
         thread_id = self._get_usable_thread()
         if thread_id is not None:
@@ -220,7 +257,7 @@ class DownloadKit(object):
         else:
             self._waiting_list.put(mission)
 
-    def _run(self, ID: int, mission: Mission) -> None:
+    def _run(self, ID: int, mission: BaseTask) -> None:
         """线程函数                                 \n
         :param ID: 线程id
         :param mission: 任务对象，Mission或Task
@@ -250,27 +287,13 @@ class DownloadKit(object):
         """
         return self._missions[mission_or_id] if isinstance(mission_or_id, int) else mission_or_id
 
-    def get_failed_missions(self, save_to: Union[str, Path] = None) -> list:
-        """返回失败任务列表，可保存到文件，支持csv、xlsx、txt、json格式     \n
-        :param save_to: 保存文件的路径
-        :return: 失败任务列表
-        """
-        lst = [i for i in self._missions.values() if i.result is False]
-        if save_to:
-            lst = [{'url': i.data.url,
-                    'path': i.data.goal_path,
-                    'rename': i.data.rename,
-                    'post_data': i.data.post_data,
-                    'kwargs': i.data.kwargs}
-                   for i in lst]
-            r = Recorder(save_to, cache_size=0)
-            r.add_data(lst)
-            r.record()
-        return lst
+    def get_failed_missions(self) -> list:
+        """返回失败任务列表"""
+        return [i for i in self._missions.values() if i.result is False]
 
     def wait(self,
              mission: Union[int, Mission] = None,
-             show: bool = True,
+             show: bool = False,
              timeout: float = None) -> Union[tuple, None]:
         """等待所有或指定任务完成                                    \n
         :param mission: 任务对象或任务id，为None时等待所有任务结束
@@ -287,13 +310,13 @@ class DownloadKit(object):
                 self.show(False)
             else:
                 t1 = perf_counter()
-                while self.is_running() or (perf_counter() - t1 < timeout or timeout == 0):
+                while self.is_running or (perf_counter() - t1 < timeout or timeout == 0):
                     sleep(0.1)
 
     def cancel(self):
         """取消所有等待中或执行中的任务"""
         for m in self._missions.values():
-            m.cancel(False)
+            m.cancel()
 
     def show(self, asyn: bool = True, keep: bool = False) -> None:
         """实时显示所有线程进度                 \n
@@ -314,14 +337,14 @@ class DownloadKit(object):
             Thread(target=self._stop_show).start()
 
         t1 = perf_counter()
-        while not self._stop_printing and (keep or self.is_running() or perf_counter() - t1 < wait):
+        while not self._stop_printing and (keep or self.is_running or perf_counter() - t1 < wait):
             print(f'\033[K', end='')
             print(f'等待任务数：{self._waiting_list.qsize()}')
             for k, v in self._threads.items():
                 m = v['mission'] if v else None
                 if m:
-                    rate = m.parent.rate if isinstance(m, Task) else m.rate if m else ''
-                    path = f'M{m.mid} {rate}% {m}'
+                    items = (m.mission.rate, m.mid) if isinstance(m, Task) else (m.rate, m.id)
+                    path = f'M{items[1]} {items[0]}% {m}'
                 else:
                     path = '空闲'
                 print(f'\033[K', end='')
@@ -337,109 +360,6 @@ class DownloadKit(object):
 
         print()
 
-    def _download(self, mission: Mission, thread_id: int) -> None:
-        """此方法是执行下载的线程方法，用于根据任务下载文件     \n
-        :param mission: 下载任务对象
-        :param thread_id: 线程号
-        :return: None
-        """
-        if mission.state in ('cancel', 'done'):
-            mission.state = 'done'
-            return
-
-        file_url = mission.data.url
-        post_data = mission.data.post_data
-        kwargs = mission.data.kwargs
-
-        if isinstance(mission, Task):
-            kwargs = CaseInsensitiveDict(kwargs)
-            if 'headers' not in kwargs:
-                kwargs['headers'] = {'Range': f"bytes={mission.range[0]}-{mission.range[1]}"}
-            else:
-                kwargs['headers']['Range'] = f"bytes={mission.range[0]}-{mission.range[1]}"
-
-            mode = 'post' if post_data is not None or kwargs.get('json', None) else 'get'
-            r, inf = self._connect(file_url, mode=mode, data=post_data, **kwargs)
-
-            if r:
-                _do_download(r, mission, False, self._lock)
-            else:
-                _set_result(mission, False, inf, 'done')
-                mission.parent.cancel()
-                _set_result(mission.parent, False, inf, 'done')
-
-            return
-
-        # ===================开始处理mission====================
-        mission.info = '下载中'
-        mission.state = 'running'
-
-        rename = mission.data.rename
-        goal_path = mission.data.goal_path
-        file_exists = mission.data.file_exists
-        split = mission.data.split
-
-        goal_Path = Path(goal_path)
-        # 按windows规则去除路径中的非法字符
-        goal_path = goal_Path.anchor + sub(r'[*:|<>?"]', '', goal_path.lstrip(goal_Path.anchor)).strip()
-        goal_Path = Path(goal_path).absolute()
-        goal_Path.mkdir(parents=True, exist_ok=True)
-        goal_path = str(goal_Path)
-
-        if file_exists == 'skip' and rename and (goal_Path / rename).exists():
-            mission.file_name = rename
-            mission.path = goal_Path / rename
-            _set_result(mission, 'skip', str(mission.path), 'done')
-            return
-
-        mode = 'post' if post_data is not None or kwargs.get('json', None) else 'get'
-        r, inf = self._connect(file_url, mode=mode, data=post_data, **kwargs)
-
-        if not r:
-            mission.cancel()
-            _set_result(mission, False, inf, 'done')
-            return
-
-        # -------------------获取文件信息-------------------
-        file_info = _get_file_info(r, goal_path, rename, file_exists, self._lock)
-        file_size = file_info['size']
-        full_path = file_info['path']
-        mission.path = full_path
-        mission.file_name = full_path.name
-        mission.size = file_size
-
-        if file_info['skip']:
-            _set_result(mission, 'skip', full_path, 'done')
-            return
-
-        if not r:
-            _set_result(mission, False, inf, 'done')
-            return
-
-        # -------------------设置分块任务-------------------
-        first = False
-        if split and file_size and file_size > self.block_size and r.headers.get('Accept-Ranges') == 'bytes':
-            first = True
-            chunks = [[s, min(s + self.block_size, file_size)] for s in range(0, file_size, self.block_size)]
-            chunks[-1][-1] = ''
-            chunks_len = len(chunks)
-
-            task1 = Task(mission, chunks[0], f'1/{chunks_len}')
-            mission.tasks = []
-            mission.tasks.append(task1)
-
-            for ind, chunk in enumerate(chunks[1:], 2):
-                task = Task(mission, chunk, f'{ind}/{chunks_len}')
-                mission.tasks.append(task)
-                self._run_or_wait(task)
-
-        else:  # 不分块
-            task1 = Task(mission, None, '1/1')
-            mission.tasks.append(task1)
-
-        self._threads[thread_id]['mission'] = task1
-        _do_download(r, task1, first, self._lock)
-
     def _connect(self,
                  url: str,
                  mode: str = 'get',
@@ -510,82 +430,187 @@ class DownloadKit(object):
         input()
         self._stop_printing = True
 
+    def _when_mission_done(self, mission: Mission) -> None:
+        """当任务完成时执行的操作                     \n
+        :param mission: 完结的任务
+        :return: None
+        """
+        if self.set_print().log_mode == 'all' or (self.set_print().log_mode == 'fail' and mission.result is False):
+            print(f'{mission.data.url}\n{mission.result}\n{mission.info}\n')
 
-def _do_download(r: Response, task: Task, first: bool = False, lock: Lock = None):
+        if self.set_log().log_mode == 'all' or (self.set_log().log_mode == 'fail' and mission.result is False):
+            data = {'url': mission.data.url,
+                    'path': mission.data.goal_path,
+                    'rename': mission.data.rename,
+                    'post_data': mission.data.post_data,
+                    'kwargs': mission.data.kwargs}
+            self._logger.add_data(data)
+
+    def _download(self,
+                  mission_or_task: Union[Mission, Task],
+                  thread_id: int) -> None:
+        """此方法是执行下载的线程方法，用于根据任务下载文件     \n
+        :param mission_or_task: 下载任务对象
+        :param thread_id: 线程号
+        :return: None
+        """
+        if mission_or_task.is_done:
+            return
+        if mission_or_task.state == 'cancel':
+            mission_or_task.state = 'done'
+            return
+
+        file_url = mission_or_task.data.url
+        post_data = mission_or_task.data.post_data
+        kwargs = mission_or_task.data.kwargs
+
+        if isinstance(mission_or_task, Task):
+            task = mission_or_task
+            kwargs = CaseInsensitiveDict(kwargs)
+            if 'headers' not in kwargs:
+                kwargs['headers'] = {'Range': f"bytes={task.range[0]}-{task.range[1]}"}
+            else:
+                kwargs['headers']['Range'] = f"bytes={task.range[0]}-{task.range[1]}"
+
+            mode = 'post' if post_data is not None or kwargs.get('json', None) else 'get'
+            r, inf = self._connect(file_url, mode=mode, data=post_data, **kwargs)
+
+            if r:
+                _do_download(r, task, False)
+            else:
+                task.set_done(False, inf)
+
+            return
+
+        # ===================开始处理mission====================
+        mission = mission_or_task
+        mission.info = '下载中'
+        mission.state = 'running'
+
+        rename = mission.data.rename
+        goal_path = mission.data.goal_path
+        file_exists = mission.data.file_exists
+        split = mission.data.split
+
+        goal_Path = Path(goal_path)
+        # 按windows规则去除路径中的非法字符
+        goal_path = goal_Path.anchor + sub(r'[*:|<>?"]', '', goal_path.lstrip(goal_Path.anchor)).strip()
+        goal_Path = Path(goal_path).absolute()
+        goal_Path.mkdir(parents=True, exist_ok=True)
+        goal_path = str(goal_Path)
+
+        if file_exists == 'skip' and rename and (goal_Path / rename).exists():
+            mission.file_name = rename
+            mission.path = goal_Path / rename
+            mission.set_done('skip', str(mission.path))
+            return
+
+        mode = 'post' if post_data is not None or kwargs.get('json', None) else 'get'
+        r, inf = self._connect(file_url, mode=mode, data=post_data, **kwargs)
+
+        if mission.is_done:
+            return
+
+        if not r:
+            mission.break_mission(result=False, info=inf)
+            return
+
+        # -------------------获取文件信息-------------------
+        file_info = _get_file_info(r, goal_path, rename, file_exists, self._lock)
+        file_size = file_info['size']
+        full_path = file_info['path']
+        mission.path = full_path
+        mission.file_name = full_path.name
+        mission.size = file_size
+
+        if file_info['skip']:
+            mission.set_done('skip', str(mission.path))
+            return
+
+        # -------------------设置分块任务-------------------
+        first = False
+        if split and file_size and file_size > self.block_size and r.headers.get('Accept-Ranges') == 'bytes':
+            first = True
+            chunks = [[s, min(s + self.block_size, file_size)] for s in range(0, file_size, self.block_size)]
+            chunks[-1][-1] = ''
+            chunks_len = len(chunks)
+
+            task1 = Task(mission, chunks[0], f'1/{chunks_len}')
+            mission.tasks_count = chunks_len
+            mission.tasks = []
+            mission.tasks.append(task1)
+
+            for ind, chunk in enumerate(chunks[1:], 2):
+                task = Task(mission, chunk, f'{ind}/{chunks_len}')
+                mission.tasks.append(task)
+                self._run_or_wait(task)
+
+        else:  # 不分块
+            task1 = Task(mission, None, '1/1')
+            mission.tasks.append(task1)
+
+        self._threads[thread_id]['mission'] = task1
+        _do_download(r, task1, first)
+
+
+def _do_download(r: Response, task: Task, first: bool = False):
     """执行下载任务                                    \n
     :param r: Response对象
     :param task: 任务
     :param first: 是否第一个分块
-    :param lock: 线程锁
     :return: None
     """
-    if task.state in ('cancel', 'done'):
-        task.state = 'done'
+    if task.is_done or task.mission.is_done:
         return
 
-    task.state = 'running'
-    task.info = '下载中'
-
-    while True:  # 争夺文件读写权限
-        try:
-            f = open(task.path, 'rb+')
-            break
-        except PermissionError:
-            sleep(.2)
+    task.set_states(result=None, info='下载中', state='running')
+    block_size = 65536  # 64k
+    result = None
 
     try:
         if first:  # 分块时第一块
-            # f.write(next(r.iter_content(chunk_size=task.range[1])))
-            blocks = int(task.range[1] / 65536)
-            remainder = task.range[1] % 65536
-            r_content = r.iter_content(chunk_size=65536)
-            for _ in range(blocks):
-                if task.state in ('cancel', 'done'):
-                    break
-                f.write(next(r_content))
-
-            if remainder and task.state not in ('cancel', 'done'):
-                f.write(next(r_content)[:remainder])
+            r_content = r.iter_content(chunk_size=task.range[1])
+            task.add_data(next(r_content), 0)
 
         else:
-            if task.range:
-                f.seek(task.range[0])
-            for chunk in r.iter_content(chunk_size=65536):
-                if task.state in ('cancel', 'done'):
-                    break
-                if chunk:
-                    f.write(chunk)
+            if task.range is None:  # 不分块
+                for chunk in r.iter_content(chunk_size=block_size):
+                    if task.state in ('cancel', 'done'):
+                        result = 'cancel'
+                        break
+                    if chunk:
+                        task.add_data(chunk, None)
+
+            elif task.range[1] == '':  # 结尾的数据块
+                begin = task.range[0]
+                for chunk in r.iter_content(chunk_size=block_size):
+                    if task.state in ('cancel', 'done'):
+                        result = 'cancel'
+                        break
+                    if chunk:
+                        task.add_data(chunk, seek=begin)
+                        begin += len(chunk)
+
+            else:  # 有始末数字的数据块
+                begin, end = task.range
+                num = (end - begin) // block_size
+                for ind, chunk in enumerate(r.iter_content(chunk_size=block_size), 1):
+                    if task.state in ('cancel', 'done'):
+                        result = 'cancel'
+                        break
+                    if chunk:
+                        task.add_data(chunk, seek=begin)
+                        if ind <= num:
+                            begin += block_size
 
     except Exception as e:
-        success, info = False, f'下载失败。{r.status_code} {e}'
+        result, info = False, f'下载失败。{r.status_code} {e}'
 
     else:
-        success, info = 'success', str(task.path)
+        result = 'success' if result is None else result
+        info = str(task.path)
 
     finally:
-        f.close()
         r.close()
 
-    task.state = 'done'
-    task.result = success
-    task.info = info
-    mission = task.parent
-
-    if not success:
-        mission.cancel()
-        mission.result = success
-        mission.info = info
-
-    if mission.is_done() and mission.is_success() is False:
-        with lock:
-            mission.del_file()
-
-
-def _set_result(mission: Mission,
-                res: Union[bool, None, str],
-                info: str,
-                state: str) -> None:
-    """设置任务结果值"""
-    mission.result = res
-    mission.info = info
-    mission.state = state
+    task.set_done(result=result, info=info)
